@@ -51,12 +51,18 @@
 
 #include "debug.h"
 #include "support.h"
+#include "cert.h"
 #include "agent.h"
 
 
 /* The global agent context.  */
 static assuan_context_t agent_ctx = NULL;
 
+/* The version number of the agent.  */
+static int agent_version_major;
+static int agent_version_minor;
+
+
 
 /* Hack required for Windows.  */
 void 
@@ -66,7 +72,7 @@ gnupg_allow_set_foregound_window (pid_t pid)
     return;
 #ifdef HAVE_W32_SYSTEM
   else if (!AllowSetForegroundWindow (pid))
-    DEBUG ("AllowSetForegroundWindow(%lu) failed: %i\n",
+    DEBUG (DBG_CRIT, "AllowSetForegroundWindow(%lu) failed: %i\n",
 	   (unsigned long)pid, GetLastError ());
 #endif
 }
@@ -189,7 +195,7 @@ spawn_process_detached (const char *pgmname, const char *argv[])
               | GetPriorityClass (GetCurrentProcess ())
               | CREATE_NEW_PROCESS_GROUP
               | DETACHED_PROCESS); 
-  DEBUG ("CreateProcess(detached), path=`%s' cmdline=`%s'\n",
+  DEBUG (DBG_INFO, "CreateProcess(detached), path=`%s' cmdline=`%s'\n",
 	 pgmname, cmdline);
   if (!CreateProcess (pgmname,       /* Program to start.  */
                       cmdline,       /* Command line arguments.  */
@@ -203,16 +209,16 @@ spawn_process_detached (const char *pgmname, const char *argv[])
                       &pi            /* Returns process information.  */
                       ))
     {
-      DEBUG ("CreateProcess(detached) failed: %i\n", GetLastError ());
+      DEBUG (DBG_CRIT, "CreateProcess(detached) failed: %i\n",
+	     GetLastError ());
       free (cmdline);
       return gpg_error (GPG_ERR_GENERAL);
     }
   free (cmdline);
   cmdline = NULL;
 
-  DEBUG ("CreateProcess(detached) ready: hProcess=%p hThread=%p"
-	 " dwProcessID=%d dwThreadId=%d\n",
-	 pi.hProcess, pi.hThread,
+  DEBUG (DBG_INFO, "CreateProcess(detached) ready: hProcess=%p hThread=%p"
+	 " dwProcessID=%d dwThreadId=%d\n", pi.hProcess, pi.hThread,
 	 (int) pi.dwProcessId, (int) pi.dwThreadId);
 
   CloseHandle (pi.hThread); 
@@ -254,8 +260,9 @@ agent_connect (assuan_context_t *ctx_r)
 	  const char *agent_program;
 
           /* With no success start a new server.  */
-	  DEBUG ("no running GPG agent at %s, starting one\n", sockname);
-          
+	  DEBUG (DBG_INFO, "no running GPG agent at %s, starting one\n",
+		 sockname);
+
           agent_program = get_gpg_agent_path ();
 
 #ifdef HAVE_W32_SYSTEM
@@ -274,7 +281,7 @@ agent_connect (assuan_context_t *ctx_r)
 
             err = spawn_process_detached (agent_program, argv);
             if (err)
-              DEBUG ("failed to start agent `%s': %s\n",
+              DEBUG (DBG_CRIT, "failed to start agent `%s': %s\n",
 		     agent_program, gpg_strerror (err));
             else
               {
@@ -323,7 +330,7 @@ agent_connect (assuan_context_t *ctx_r)
 
       if (!(ptr = strchr (infostr, PATHSEP_C)) || ptr == infostr)
 	{
-	  DEBUG ("malformed GPG_AGENT_INFO environment variable");
+	  DEBUG (DBG_CRIT, "malformed GPG_AGENT_INFO environment variable");
 	  free (infostr);
 	  force_pipe_server = 1;
 	  goto restart;
@@ -336,7 +343,7 @@ agent_connect (assuan_context_t *ctx_r)
       protocol_version = *ptr ? atoi (ptr + 1) : 0;
       if (protocol_version != 1)
 	{
-	  DEBUG ("GPG agent protocol version '%d' not supported",
+	  DEBUG (DBG_CRIT, "GPG agent protocol version '%d' not supported",
 		 protocol_version);
 	  free (infostr);
 	  force_pipe_server = 1;
@@ -347,7 +354,7 @@ agent_connect (assuan_context_t *ctx_r)
       free (infostr);
       if (err)
 	{
-	  DEBUG ("cannot connect to GPG agent: %s", gpg_strerror (err));
+	  DEBUG (DBG_CRIT, "cannot connect to GPG agent: %s", gpg_strerror (err));
 	  force_pipe_server = 1;
 	  goto restart;
 	}
@@ -355,17 +362,20 @@ agent_connect (assuan_context_t *ctx_r)
 
   if (err)
     {
-      DEBUG ("cannot connect to GPG agent: %s", gpg_strerror (err));
+      DEBUG (DBG_CRIT, "cannot connect to GPG agent: %s", gpg_strerror (err));
       return gpg_error (GPG_ERR_NO_AGENT);
     }
 
+  if (_scute_debug_flags & DBG_ASSUAN)
+    assuan_set_log_stream (*ctx_r, _scute_debug_stream);
+
   return 0;
 }
 
 
 /* This is the default inquiry callback.  It mainly handles the
    Pinentry notifications.  */
-static int
+static gpg_error_t
 default_inq_cb (void *opaque, const char *line)
 {
   (void)opaque;
@@ -376,13 +386,12 @@ default_inq_cb (void *opaque, const char *line)
       /* We do not pass errors to avoid breaking other code.  */
     }
   else
-    DEBUG ("ignoring gpg-agent inquiry `%s'\n", line);
+    DEBUG (DBG_CRIT, "ignoring gpg-agent inquiry `%s'\n", line);
 
   return 0;
 }
 
 
-
 /* Send a simple command to the agent.  */
 static gpg_error_t 
 agent_simple_cmd (assuan_context_t ctx, const char *fmt, ...)
@@ -402,11 +411,32 @@ agent_simple_cmd (assuan_context_t ctx, const char *fmt, ...)
   err = assuan_transact (ctx, optstr, NULL, NULL, default_inq_cb,
 			 NULL, NULL, NULL);
   if (err)
-    DEBUG ("gpg-agent command '%s' failed: %s", optstr, gpg_strerror (err));
+    DEBUG (DBG_CRIT, "gpg-agent command '%s' failed: %s", optstr,
+	   gpg_strerror (err));
   free (optstr);
       
   return err;
 }
+
+
+/* Read and stroe the agent's version number.  */
+static gpg_error_t
+read_version_cb (void *opaque, const void *buffer, size_t length)
+{
+  char version[20];
+  const char *s;
+  
+  if (length > sizeof (version) -1)
+    length = sizeof (version) - 1;
+  strncpy (version, buffer, length);
+  version[length] = 0;
+
+  agent_version_major = atoi (version);
+  s = strchr (version, '.');
+  agent_version_minor = s? atoi (s+1) : 0;
+
+  return 0;
+}
   
 
 /* Configure the GPG agent at connection CTX.  */
@@ -512,6 +542,15 @@ agent_configure (assuan_context_t ctx)
   else if (err)
     return err;
 
+  err = assuan_transact (ctx, "GETINFO version",
+                         read_version_cb, NULL,
+                         NULL, NULL, NULL, NULL);
+  if (gpg_err_code (err) == GPG_ERR_UNKNOWN_OPTION)
+    err = 0;
+  else if (err)
+    return err;
+
+
   return err;
 }
 
@@ -525,10 +564,11 @@ scute_agent_initialize (void)
 
   if (agent_ctx)
     {
-      DEBUG ("GPG Agent connection already established");
+      DEBUG (DBG_CRIT, "GPG Agent connection already established");
       return 0;
     }
 
+  DEBUG (DBG_INFO, "Establishing connection to gpg-agent");
   err = agent_connect (&agent_ctx);
   if (err)
     return err;
@@ -540,6 +580,15 @@ scute_agent_initialize (void)
   return err;
 }
 
+
+int
+scute_agent_get_agent_version (int *minor)
+{
+  *minor = agent_version_minor;
+  return agent_version_major;
+}
+
+
 
 /* Return a new malloced string by unescaping the string S.  Escaping
    is percent escaping and '+'/space mapping.  A binary nul will
@@ -811,6 +860,38 @@ scute_agent_learn (struct agent_card_info_s *info)
 }
 
 
+
+static gpg_error_t
+geteventcounter_status_cb (void *opaque, const char *line)
+{
+  int *result = opaque;
+  const char *keyword = line;
+  int keywordlen;
+
+  for (keywordlen=0; *line && !spacep (line); line++, keywordlen++)
+    ;
+  while (spacep (line))
+    line++;
+
+  if (keywordlen == 12 && !memcmp (keyword, "EVENTCOUNTER", keywordlen))
+    {
+      static int any_count;
+      static unsigned int last_count;
+      unsigned int count;
+
+      if (sscanf (line, "%*u %*u %u ", &count) == 1)
+        {
+          if (any_count && last_count != count)
+            *result = 1;
+          any_count = 1;
+          last_count = count;
+        }
+    }
+  
+  return 0;
+}
+
+
 static gpg_error_t
 read_status_cb (void *opaque, const void *buffer, size_t length)
 {
@@ -831,15 +912,33 @@ read_status_cb (void *opaque, const void *buffer, size_t length)
 gpg_error_t
 scute_agent_check_status (void)
 {
+  static char last_flag;
   gpg_error_t err;
+  int any = 0;
   char flag = '-';
 
-  err = assuan_transact (agent_ctx, "SCD GETINFO status",
-			 read_status_cb, &flag, default_inq_cb,
-			 NULL, NULL, NULL);
+  /* First we look at the eventcounter to see if anything happened at
+     all.  This is a low overhead function which won't even clutter a
+     gpg-agent log file.  There is no need for error checking here. */
+  if (last_flag)
+    assuan_transact (agent_ctx, "GETEVENTCOUNTER",
+                     NULL, NULL,
+                     NULL, NULL,
+                     geteventcounter_status_cb, &any);
+
+  if (any || !last_flag)
+    {
+      err = assuan_transact (agent_ctx, "SCD GETINFO status",
+                             read_status_cb, &flag,
+                             default_inq_cb, NULL,
+                             NULL, NULL);
+      if (err)
+        return err;
+      last_flag = flag;
+    }
+  else
+    flag = last_flag;
 
-  if (err)
-    return err;
 
   if (flag == 'r')
     return gpg_error (GPG_ERR_CARD_REMOVED);
@@ -848,7 +947,8 @@ scute_agent_check_status (void)
 }
 
 
-#define MAX_SIGNATURE_LEN 256
+/* Enough space to hold a 2048 bit RSA signature in an S-expression.  */
+#define MAX_SIGNATURE_LEN 350
 
 struct signature
 {
@@ -862,7 +962,10 @@ pksign_cb (void *opaque, const void *buffer, size_t length)
   struct signature *sig = opaque;
 
   if (sig->len + length > MAX_SIGNATURE_LEN)
-    return gpg_error (GPG_ERR_BAD_DATA);
+    {
+      DEBUG (DBG_INFO, "maximum signature length exceeded");
+      return gpg_error (GPG_ERR_BAD_DATA);
+    }
 
   memcpy (&sig->data[sig->len], buffer, length);
   sig->len += length;
@@ -871,11 +974,13 @@ pksign_cb (void *opaque, const void *buffer, size_t length)
 }
 
 
-#define SIG_PREFIX "(7:sig-val(3:rsa(1:s128:"
+#define SIG_PREFIX   "(7:sig-val(3:rsa(1:s128:"
+#define SIG_PREFIX_2 "(7:sig-val(3:rsa(1:s256:"
 #define SIG_PREFIX_LEN (sizeof (SIG_PREFIX) - 1)
 #define SIG_POSTFIX ")))"
 #define SIG_POSTFIX_LEN (sizeof (SIG_POSTFIX) - 1)
 #define SIG_LEN 128
+#define SIG_LEN_2 256
 
 /* Call the agent to learn about a smartcard.  */
 gpg_error_t
@@ -896,7 +1001,8 @@ scute_agent_sign (char *grip, unsigned char *data, int len,
 
   if (sig_result == NULL)
     {
-      *sig_len = SIG_LEN;
+      /* FIXME:  We return the largest supported size - is that correct?  */
+      *sig_len = SIG_LEN_2;
       return 0;
     }
 
@@ -913,10 +1019,10 @@ scute_agent_sign (char *grip, unsigned char *data, int len,
     return err;
 
   for (i = 0; i < len; i++)
-    sprintf (&pretty_data[2 * i], "%02X", data[i]);
+    snprintf (&pretty_data[2 * i], 3, "%02X", data[i]);
   pretty_data[2 * len] = '\0';
 
-  snprintf (cmd, sizeof (cmd), "sethash --hash=tls-md5sha1 %s", pretty_data);
+  snprintf (cmd, sizeof (cmd), "SETHASH --hash=tls-md5sha1 %s", pretty_data);
   err = assuan_transact (agent_ctx, cmd, NULL, NULL, default_inq_cb,
 			 NULL, NULL, NULL);
   if (err)
@@ -927,23 +1033,38 @@ scute_agent_sign (char *grip, unsigned char *data, int len,
   if (err)
     return err;
 
-  if (sig.len != SIG_PREFIX_LEN + SIG_LEN + SIG_POSTFIX_LEN)
-    return gpg_error (GPG_ERR_BAD_SIGNATURE);
-  if (memcmp (sig.data, SIG_PREFIX, SIG_PREFIX_LEN))
-    return gpg_error (GPG_ERR_BAD_SIGNATURE);
-  if (memcmp (sig.data + sig.len - SIG_POSTFIX_LEN,
-	      SIG_POSTFIX, SIG_POSTFIX_LEN))
-    return gpg_error (GPG_ERR_BAD_SIGNATURE);
-
-  memcpy (sig_result, sig.data + SIG_PREFIX_LEN, SIG_LEN);
-  *sig_len = SIG_LEN;
+  /* FIXME: we need a real parser to cope with all kind of S-expressions.  */
+  if (sig.len == SIG_PREFIX_LEN + SIG_LEN_2 + SIG_POSTFIX_LEN)
+    {
+      if (memcmp (sig.data, SIG_PREFIX_2, SIG_PREFIX_LEN))
+        return gpg_error (GPG_ERR_BAD_SIGNATURE);
+      if (memcmp (sig.data + sig.len - SIG_POSTFIX_LEN,
+                  SIG_POSTFIX, SIG_POSTFIX_LEN))
+        return gpg_error (GPG_ERR_BAD_SIGNATURE);
+      memcpy (sig_result, sig.data + SIG_PREFIX_LEN, SIG_LEN_2);
+      *sig_len = SIG_LEN_2;
+    }
+  else
+    {
+      if (sig.len != SIG_PREFIX_LEN + SIG_LEN + SIG_POSTFIX_LEN)
+        return gpg_error (GPG_ERR_BAD_SIGNATURE);
+      if (memcmp (sig.data, SIG_PREFIX, SIG_PREFIX_LEN))
+        return gpg_error (GPG_ERR_BAD_SIGNATURE);
+      if (memcmp (sig.data + sig.len - SIG_POSTFIX_LEN,
+                  SIG_POSTFIX, SIG_POSTFIX_LEN))
+        return gpg_error (GPG_ERR_BAD_SIGNATURE);
+      memcpy (sig_result, sig.data + SIG_PREFIX_LEN, SIG_LEN);
+      *sig_len = SIG_LEN;
+    }
+  
   
   return 0;
 }
 
 
 /* Determine if FPR is trusted.  */
-gpg_error_t scute_agent_is_trusted (char *fpr, bool *is_trusted)
+gpg_error_t
+scute_agent_is_trusted (char *fpr, bool *is_trusted)
 {
   gpg_error_t err;
   bool trusted = false;
@@ -961,16 +1082,102 @@ gpg_error_t scute_agent_is_trusted (char *fpr, bool *is_trusted)
   return 0;
 }
 
+
+#define GET_CERT_INIT_SIZE 2048
+
+struct get_cert_s
+{
+  unsigned char *cert_der;
+  int cert_der_len;
+  int cert_der_size;
+};
+
+
+gpg_error_t
+get_cert_data_cb (void *opaque, const void *data, size_t data_len)
+{
+  struct get_cert_s *cert_s = opaque;
+  gpg_error_t err;
+  int needed_size;
+
+  needed_size = cert_s->cert_der_len + data_len;
+  if (needed_size > cert_s->cert_der_size)
+    {
+      unsigned char *new_cert_der;
+      int new_cert_der_size = cert_s->cert_der_size;
+
+      if (new_cert_der_size == 0)
+	new_cert_der_size = GET_CERT_INIT_SIZE;
+      while (new_cert_der_size < needed_size)
+	new_cert_der_size *= 2;
+
+      if (cert_s->cert_der == NULL)
+	new_cert_der = malloc (new_cert_der_size);
+      else
+	new_cert_der = realloc (cert_s->cert_der, new_cert_der_size);
+
+      if (new_cert_der == NULL)
+	return gpg_error_from_syserror ();
+
+      cert_s->cert_der = new_cert_der;
+      cert_s->cert_der_size = new_cert_der_size;
+    }
+
+  memcpy (cert_s->cert_der + cert_s->cert_der_len, data, data_len);
+  cert_s->cert_der_len += data_len;
+
+  return 0;
+}
+
+
+/* Try to get certificate for key numer NO.  */
+gpg_error_t
+scute_agent_get_cert (int no, struct cert *cert)
+{
+  gpg_error_t err;
+  char cmd[150];
+  struct get_cert_s cert_s;
+
+  cert_s.cert_der = NULL;
+  cert_s.cert_der_len = 0;
+  cert_s.cert_der_size = 0;
+
+  snprintf (cmd, sizeof (cmd), "SCD READCERT OPENPGP.%i", no);
+  err = assuan_transact (agent_ctx, cmd, get_cert_data_cb, &cert_s,
+			 NULL, NULL, NULL, NULL);
+  /* Just to be safe... */
+  if (!err && cert_s.cert_der_len <= 16)
+    {
+      DEBUG (DBG_INFO, "bad card certificate rejected");
+      err = gpg_error (GPG_ERR_BAD_CERT);
+    }
+  if (err)
+    {
+      if (cert_s.cert_der)
+	free (cert_s.cert_der);
+      return err;
+    }
+
+  DEBUG (DBG_INFO, "got certificate from card with length %i",
+	 cert_s.cert_der_len);
+
+  cert->cert_der = cert_s.cert_der;
+  cert->cert_der_len = cert_s.cert_der_len;
+
+  return 0;
+}
+
 
 void
 scute_agent_finalize (void)
 {
   if (!agent_ctx)
     {
-      DEBUG ("no GPG Agent connection established");
+      DEBUG (DBG_CRIT, "no GPG Agent connection established");
       return;
     }
 
+  DEBUG (DBG_INFO, "releasing agent context");
   assuan_disconnect (agent_ctx);
   agent_ctx = NULL;
 }
